<?php

namespace IQBall\Core\Gateway;
use IQBall\Core\Connection;
use IQBall\Core\Data\PersonalSpace\PersonalSpaceFolder;
use IQBall\Core\Data\TacticInfo;
use PDO;
use IQBall\Core\Data\CourtType;

class PersonalSpaceGateway {
    private Connection $con;

    public function __construct(Connection $con) {
        $this->con = $con;
    }

    public function addFolder(string $folderName, int $ownerId, int $parentId): void {
        $this->con->exec("INSERT INTO TacticFolder(name,owner,tactic_folder_parent) VALUES(:name,:owner,:parent)",
            [
                "name" => [$folderName, \PDO::PARAM_STR],
                "owner" => [$ownerId, \PDO::PARAM_INT],
                "parent" => [$parentId, \PDO::PARAM_INT]
            ]
        );
    }

    /**
     * Return all the tactic of a specific folder
     * If the folder's id is 0, we return all tactics at the root
     * @param int $accountId
     * @param int $folderId
     * @return array
     */
    public function getFolder(int $accountId, int $folderId): array {
        if ($folderId == 0) {
            $folders = $this->getFolderFromRoot($accountId);
        } else {
            $folders = $this->getFolderFromFolder($accountId, $folderId);
        }
        return $folders;
    }

    public function getTactic(int $accountId, int $folderId):array{
        if ($folderId == 0) {
            $tactics = $this->getTacticFromRoot($accountId);
        } else {
            $tactics = $this->getTacticFromFolder($accountId, $folderId);
        }
        return $tactics;
    }


    private function getTacticFromRoot(int $accountId): array {
        $result = $this->con->fetch(
        "SELECT t.*
            FROM Tactic t
            WHERE t.owner = :ownerId AND t.id NOT IN (SELECT id_tactic FROM TacticFolderLink)",
            ["ownerId" => [$accountId, PDO::PARAM_INT]]
        );
        return array_map(fn($row) => new TacticInfo($row['id'],$row['name'],$row['creation_date'],$row['owner'],CourtType::fromName($row['court_type']),$row['content']),$result);
    }

    private function getFolderFromRoot(int $accountId): array {
        $result = $this->con->fetch(
        "SELECT *
            FROM TacticFolder
            WHERE owner = :owner AND tactic_folder_parent IS NULL",
            ["ownerId" => [$accountId, PDO::PARAM_INT]]
        );
        var_dump($result);
        return array_map(fn($row) => new PersonalSpaceFolder($row['id'],$row['name']),$result);
    }

    private function getTacticFromFolder(int $accountId, int $folderId): array {
        $result = $this->con->fetch(
        "SELECT t.*
            FROM Tactic t, TacticFolderLink tfl
            WHERE t.owner = :ownerId AND tfl.id_tactic = :folderId",
            [
                "ownerId" => [$accountId, PDO::PARAM_INT],
                "folderId" => [$folderId, PDO::PARAM_INT]
            ]
        );
        return array_map(fn($row) => new TacticInfo($row['id'],$row['name'],$row['creation_date'],$row['owner'],CourtType::fromName($row['court_type']),$row['content']),$result);

    }

    private function getFolderFromFolder(int $accountId, int $folderId): array {
        $result = $this->con->fetch(
            "SELECT *
                FROM TacticFolder
                WHERE owner = :ownerId AND tactic_folder_parent = :folderId",
            [
                "ownerId" => [$accountId, PDO::PARAM_INT],
                "folderId" => [$folderId, PDO::PARAM_INT]
            ]
        );
        return array_map(fn($row) => new PersonalSpaceFolder($row['id'],$row['name']),$result);
    }
}